/// <reference types="node" />
import { IRouter, NextFunction, Request, Response } from "express";
import { K8SHealthStatus } from "./enum-k8s-health-status";
import { DisposableSingletonContainer } from "./disposable-singleton-container";
declare module 'express' {
    interface Express {
        [Symbol.asyncDispose]?: () => Promise<void>;
    }
}
export declare class ApplicationBuilder {
    applicationName: string;
    swaggerDocument: any;
    private readonly currentProcess;
    private readonly exitSignals;
    private readonly container;
    healthStatus: K8SHealthStatus;
    private applicationPort;
    private healthPort;
    private appMiddlewares;
    private appRouters;
    private helmetMiddleware;
    private bodyParserUrlEncodingMiddleware;
    private bodyParserJsonMiddleware;
    private catchAllErrorResponseTransformer;
    private readonly exitHandler;
    constructor(applicationName?: string, swaggerDocument?: any, currentProcess?: NodeJS.Process, exitSignals?: NodeJS.Signals[], container?: DisposableSingletonContainer);
    overrideAppPort(port: number): ApplicationBuilder;
    overrideHealthPort(port: number): ApplicationBuilder;
    overrideHelmetConfiguration(helmet: (request: Request, response: Response, next: NextFunction) => void): ApplicationBuilder;
    overrideBodyParserUrlEncodingConfiguration(bodyParserUrlEncoding: (request: Request, response: Response, next: NextFunction) => void): ApplicationBuilder;
    overrideBodyParserJsonConfiguration(bodyParserJson: (request: Request, response: Response, next: NextFunction) => void): ApplicationBuilder;
    overrideCatchAllErrorResponseTransformer(transformer: (request: Request, error: unknown) => unknown): ApplicationBuilder;
    registerApplicationMiddleware(middleware: (request: Request, response: Response, next: NextFunction) => Promise<void>): ApplicationBuilder;
    registerApplicationRoutes(path: string, router: IRouter): ApplicationBuilder;
    changeHealthStatus(status: K8SHealthStatus): void;
    start(): Promise<void>;
    [Symbol.asyncDispose](): Promise<void>;
    private appExpressListen;
    private healthExpressListen;
    private checkHealthStatus;
    private errorHandler;
}
