"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.DisposableSingletonContainer = void 0;
const bootstrap_constructor_1 = require("./bootstrap-constructor");
class DisposableSingletonContainer {
    constructor(singletonContainer = new Map(), disposeSequenceMap = new Map(), disposeSequence = 0, bootstrap = new bootstrap_constructor_1.BootstrapConstructor()) {
        this.singletonContainer = singletonContainer;
        this.disposeSequenceMap = disposeSequenceMap;
        this.disposeSequence = disposeSequence;
        this.bootstrap = bootstrap;
    }
    createInstance(name, typeConstructor, constructorArguments, disposeSequence) {
        if (!this.singletonContainer.has(name)) {
            const newInstance = this.bootstrap.createInstance(typeConstructor);
            this.disposeSequence++;
            disposeSequence = disposeSequence || this.disposeSequence;
            const existingMembers = this.disposeSequenceMap.get(disposeSequence) || new Set();
            existingMembers.add(name);
            this.disposeSequenceMap.set(disposeSequence, existingMembers);
            this.singletonContainer.set(name, newInstance);
        }
        return this.singletonContainer.get(name);
    }
    createInstanceWithoutConstructor(name, typeConstructor, constructorArguments, disposeSequence) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.singletonContainer.has(name)) {
                const newInstance = yield this.bootstrap.createAsyncInstanceWithoutConstructor(typeConstructor);
                this.disposeSequence++;
                disposeSequence = disposeSequence || this.disposeSequence;
                const existingMembers = this.disposeSequenceMap.get(disposeSequence) || new Set();
                existingMembers.add(name);
                this.disposeSequenceMap.set(disposeSequence, existingMembers);
                this.singletonContainer.set(name, newInstance);
            }
            return this.singletonContainer.get(name);
        });
    }
    disposeInstance(name) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.singletonContainer.has(name)) {
                if (this.singletonContainer.get(name)[Symbol.dispose] != null) {
                    this.singletonContainer.get(name)[Symbol.dispose]();
                }
                if (this.singletonContainer.get(name)[Symbol.asyncDispose] != null) {
                    yield this.singletonContainer.get(name)[Symbol.asyncDispose]();
                }
                this.singletonContainer.delete(name);
                this.disposeSequenceMap.forEach((setOfNames, sequence) => {
                    if (setOfNames.has(name)) {
                        setOfNames.delete(name);
                        if (setOfNames.size === 0) {
                            this.disposeSequenceMap.delete(sequence);
                        }
                    }
                });
            }
        });
    }
    disposeAll() {
        return __awaiter(this, void 0, void 0, function* () {
            //Sort descending order of dispose sequence
            const sortedDisposeSequence = Array.from(this.disposeSequenceMap.keys()).sort((a, b) => b - a);
            for (const sequence of sortedDisposeSequence) {
                for (const instanceName of (this.disposeSequenceMap.get(sequence) || new Set())) {
                    yield this.disposeInstance(instanceName);
                }
            }
        });
    }
}
exports.DisposableSingletonContainer = DisposableSingletonContainer;
//# sourceMappingURL=disposable-singleton-container.js.map